@@ -8,20 +8,19 @@ from json import JSONDecodeError
 from typing import Any, List, Optional
 
 import requests
-from requests import Response
 
-from kodexa.model import ContentObject, Document, DocumentFamily, DocumentStore, DocumentTransition, ModelStore, \
-    RemoteStore
+from kodexa.model import ContentObject, Document, DocumentFamily, DocumentStore, DocumentTransition, ModelStore, Store
 from kodexa.model.model import ModelContentMetadata
 from kodexa.stores.local import LocalModelStore, TableDataStore
 
 logger = logging.getLogger('kodexa.stores')
 
 
-class RemoteTableDataStore(RemoteStore):
+class RemoteDataStore(Store):
     """ """
 
-    def __init__(self, ref: str, columns=None):
+    def __init__(self, ref: str, columns=None, **data: Any):
+        super().__init__(**data)
         if columns is None:
             columns = []
         self.ref = ref
@@ -31,58 +30,26 @@ class RemoteTableDataStore(RemoteStore):
         """ """
         return self.ref
 
-    def get_data_objects_df(self, parent: str, query: str = "*", document_family: Optional[DocumentFamily] = None,
-                            include_id=False):
+    def get_parent_df(self, parent: str, query: str = "*", document_family: Optional[DocumentFamily] = None):
         """
 
         Args:
           parent (str): The parent taxon (/ is root)
           query (str): A query to limit the results (Defaults to *)
           document_family (Optional[DocumentFamily): Optionally the document family to limit results to
-          include_id (bool): Include the id in the data object
         Returns:
 
         """
         import pandas as pd
 
-        data_objects = self.get_data_objects(parent, query, document_family)
+        table_result = self.get_parent(parent, query, document_family)
+        return pd.DataFrame(table_result['rows'], columns=table_result['columns'])
 
-        table_result = {
-            'rows': [],
-            'columns': [],
-            'column_headers': []
-        }
-
-        for data_object in data_objects:
-            if len(table_result['columns']) == 0:
-                if include_id:
-                    table_result['column_headers'].append('Data Object ID')
-                    table_result['columns'].append('data_object_id')
-                for taxon in data_object['taxon']['children']:
-                    if not taxon['group']:
-                        table_result['column_headers'].append(taxon['label'])
-                        table_result['columns'].append(taxon['name'])
-
-            new_row = []
-            for column in table_result['columns']:
-                column_value = None
-                if include_id:
-                    if column == 'data_object_id':
-                        column_value = data_object['id']
-                for attribute in data_object['attributes']:
-                    if attribute['tag'] == column:
-                        column_value = attribute['stringValue']
-                new_row.append(column_value)
-
-            table_result['rows'].append(new_row)
-
-        return pd.DataFrame(table_result['rows'], columns=table_result['column_headers'])
-
-    def get_data_objects(self, path: str, query: str = "*", document_family: Optional[DocumentFamily] = None):
+    def get_parent(self, parent: str, query: str = "*", document_family: Optional[DocumentFamily] = None):
         """
 
         Args:
-          path (str): The path to the data object
+          parent (str): The parent taxon (/ is root)
           query (str): A query to limit the results (Default *)
           document_family (Optional[DocumentFamily): Optionally the document family to limit results to
         Returns:
@@ -91,45 +58,47 @@ class RemoteTableDataStore(RemoteStore):
 
         # We need to get the first set of rows,
         rows: List = []
-        row_response = self.get_data_objects_page_request(path, 1, document_family=document_family)
+        row_response = self.get_parent_page_request(parent, 1, document_family=document_family)
 
         # lets work out the last page
         rows = rows + row_response['content']
         total_pages = row_response['totalPages']
 
         for page in range(2, total_pages):
-            row_response = self.get_data_objects_page_request(path, page, query=query, document_family=document_family)
+            row_response = self.get_parent_page_request(parent, page, query=query, document_family=document_family)
             rows = rows + row_response['content']
 
-        return rows
+        # Once we have all the rows we will then get a list of all the columns
+        # and convert this into a more nature form for structured data
 
-    def get_data_object(self, data_object_id: str):
-        from kodexa import KodexaPlatform
+        column_names: List[str] = []
+        for row in rows:
+            for key in row['data'].keys():
+                if key not in column_names:
+                    column_names.append(key)
 
-        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/dataObjects/{data_object_id}"
-        logger.info(f"Downloading a specific data object from {url}")
+        # Now lets get all the rows and make sure we put them in the same
+        # order as the columns
 
-        data_object_response = requests.get(
-            url,
-            headers={"x-access-token": KodexaPlatform.get_access_token(), "content-type": "application/json"})
+        new_rows: List[List[str]] = []
 
-        print(data_object_response)
-        if data_object_response.status_code == 200:
-            return data_object_response.json()
-        else:
-            logger.warning(
-                "Unable to get data object from remote store [" + data_object_response.text + "], response " + str(
-                    data_object_response.status_code))
-            raise Exception(
-                "Unable to get data object from remote store  [" + data_object_response.text + "], response " + str(
-                    data_object_response.status_code))
+        for row in rows:
+            new_row = []
+            for column_name in column_names:
+                new_row.append(row['data'].get(column_name, None))
+            new_rows.append(new_row)
 
-    def get_data_objects_page_request(self, path: str, page_number: int = 1, page_size=5000, query="*",
-                                      document_family: Optional[DocumentFamily] = None):
+        return {
+            "columns": column_names,
+            "rows": new_rows
+        }
+
+    def get_parent_page_request(self, parent: str, page_number: int = 1, page_size=5000, query="*",
+                                document_family: Optional[DocumentFamily] = None):
         """
 
         Args:
-          path (str): The parent taxon (/ is root)
+          parent (str): The parent taxon (/ is root)
           page_number (int):  (Default value = 1)
           page_size (int):  (Default value = 5000)
           query (str): The query to limit results (Default *)
@@ -140,11 +109,11 @@ class RemoteTableDataStore(RemoteStore):
         """
         from kodexa import KodexaPlatform
 
-        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/dataObjects"
+        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/rows"
         logger.debug(f"Downloading a specific table from {url}")
 
         # We need to go through and pull all the pages
-        params = {"path": path, "page": page_number, "pageSize": page_size, "query": query}
+        params = {"parent": parent, "page": page_number, "pageSize": page_size, "query": query}
 
         if document_family:
             params['documentFamilyId'] = document_family.id
@@ -163,7 +132,6 @@ class RemoteTableDataStore(RemoteStore):
             raise Exception("Unable to get table from remote store  [" + rows_response.text + "], response " + str(
                 rows_response.status_code))
 
-
     def add_rows(self, rows):
         """
 
@@ -175,7 +143,7 @@ class RemoteTableDataStore(RemoteStore):
         """
         from kodexa import KodexaPlatform
 
-        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/dataObjects"
+        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/rows"
         logger.debug(f"Uploading rows to store {url}")
 
         doc = requests.post(
@@ -189,18 +157,10 @@ class RemoteTableDataStore(RemoteStore):
             raise Exception("Unable to post rows to remote store [" + doc.text + "], response " + str(doc.status_code))
 
     def add(self, row):
-        """
-
-        Args:
-          row:
-
-        Returns:
-
-        """
         from kodexa import KodexaPlatform
 
-        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/dataObjects"
-        logger.debug(f"Uploading data objects to store {url}")
+        url = f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/rows"
+        logger.debug(f"Uploading rows to store {url}")
 
         row_dict = {}
         for idx, row_value in enumerate(row):
@@ -221,13 +181,8 @@ class RemoteTableDataStore(RemoteStore):
 
 
 class DataStoreHelper:
-    """A small helper that can convert a dictionary back into a store
-    type
-
-    Args:
-
-    Returns:
-
+    """
+    A small helper that can convert a dictionary back into a store
     """
 
     @staticmethod
@@ -241,7 +196,7 @@ class DataStoreHelper:
           TableDataStore, DictDataStore, or None: A TableDataStore or DictDataStore - driven from 'type' in doc_dict.
           If 'type' is not present or does not align with one of these two types, None is returend.
 
-        >>> Document.from_dict(doc_dict)
+        >>> DataStoreHelper.from_dict(doc_dict)
         """
 
         if 'type' in dict:
@@ -257,7 +212,8 @@ class DataStoreHelper:
                     return LocalDocumentStore(dict['data']['path'])
             elif 'MODEL' == dict['type']:
                 if 'ref' in dict:
-                    return RemoteModelStore(dict['ref'])
+                    from kodexa import KodexaPlatform
+                    return KodexaPlatform.get_object_instance(dict['ref'], RemoteModelStore)
                 else:
                     return LocalModelStore(dict['data']['path'])
             else:
@@ -267,64 +223,12 @@ class DataStoreHelper:
             return None
 
 
-class RemoteDocumentStore(DocumentStore, RemoteStore):
+class RemoteDocumentStore(DocumentStore):
     """
     Remote Document Stores provide you with all the capabilities of Document storage and relationships
     in an instance of the Kodexa platform
     """
 
-    def __init__(self, ref: str):
-
-        self.ref: str = ref
-
-        get_response = self._base_get(
-            f"api/stores/{self.ref.replace(':', '/')}")
-
-        if get_response is not None:
-            self.store_type = get_response.json()['storeType']
-            self.store_purpose = get_response.json()['storePurpose']
-        else:
-            # we couldn't find metadata for this store, so we're setting default values
-            self.store_type = 'DOCUMENT'
-            self.store_purpose = 'OPERATIONAL'
-
-        super().__init__(self.store_type, self.store_purpose)
-
-    def get_name(self):
-        """The name of the connector
-
-        Returns:
-            The name of the document store
-
-        """
-        return self.get_ref()
-
-    def get_ref(self) -> str:
-        """Get the reference to the store on the platform (i.e. kodexa/my-store:1.1.0)
-
-        :return: The reference
-
-        Args:
-
-        Returns:
-
-        """
-        return self.ref
-
-    def to_dict(self):
-        """
-        Convert the document store to its metadata dictionary
-
-        Returns:
-            A dictionary representing the metadata for the store
-        """
-        return {
-            "type": "DOCUMENT",
-            "ref": self.ref,
-            "storeType": self.store_type,
-            "storePurpose": self.store_purpose
-        }
-
     def delete(self, path: str):
         from kodexa import KodexaPlatform
         try:
@@ -353,7 +257,7 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
                 headers={"x-access-token": KodexaPlatform.get_access_token()})
 
             if document_family_response.status_code == 200:
-                return DocumentFamily.from_dict(document_family_response.json())
+                return DocumentFamily.parse_obj(document_family_response.json())
             else:
                 msg = "Get document family failed [" + document_family_response.text + "], response " + str(
                     document_family_response.status_code)
@@ -382,7 +286,7 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
                 files=files)
 
             if document_family_response.status_code == 200:
-                return ContentObject.from_dict(document_family_response.json())
+                return ContentObject.parse_obj(document_family_response.json())
             else:
                 msg = "Document family create failed [" + document_family_response.text + "], response " + str(
                     document_family_response.status_code)
@@ -395,13 +299,15 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
 
     def get_document_by_content_object(self, document_family: DocumentFamily, content_object: ContentObject) -> \
             Optional[Document]:
-        get_response = self._base_get(
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(
             f"api/stores/{self.ref.replace(':', '/')}/families/{document_family.id}/objects/{content_object.id}/content")
         return Document.from_kddb(get_response.content) if get_response is not None else None
 
     def get_source_by_content_object(self, document_family: DocumentFamily, content_object: ContentObject) -> \
             Any:
-        get_response = self._base_get(
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(
             f"api/stores/{self.ref.replace(':', '/')}/families/{document_family.id}/objects/{content_object.id}/content")
         return get_response.content if get_response is not None else None
 
@@ -414,11 +320,13 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
             'pageSize': page_size,
             'query': query
         }
-        get_response = self._base_get(f"api/stores/{self.ref.replace(':', '/')}/families", params=params)
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(f"api/stores/{self.ref.replace(':', '/')}/families",
+                                                       params=params)
         if get_response is not None:
             families = []
             for fam_dict in get_response.json()['content']:
-                families.append(DocumentFamily.from_dict(fam_dict))
+                families.append(DocumentFamily.parse_obj(fam_dict))
             return families
         else:
             return []
@@ -437,7 +345,7 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
                 files=files)
 
             if content_object_replace.status_code == 200:
-                return DocumentFamily.from_dict(content_object_replace.json())
+                return DocumentFamily.parse_obj(content_object_replace.json())
             else:
                 msg = "Document replace failed [" + content_object_replace.text + "], response " + str(
                     content_object_replace.status_code)
@@ -462,7 +370,7 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
                 files=files, data=data)
 
             if document_family_response.status_code == 200:
-                return DocumentFamily.from_dict(document_family_response.json())
+                return DocumentFamily.parse_obj(document_family_response.json())
             else:
                 msg = "Document family create failed [" + document_family_response.text + "], response " + str(
                     document_family_response.status_code)
@@ -486,7 +394,7 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
                 files=files)
 
             if document_family_response.status_code == 200:
-                return DocumentFamily.from_dict(document_family_response.json())
+                return DocumentFamily.parse_obj(document_family_response.json())
             else:
                 msg = "Document family create failed [" + document_family_response.text + "], response " + str(
                     document_family_response.status_code)
@@ -498,72 +406,34 @@ class RemoteDocumentStore(DocumentStore, RemoteStore):
             raise
 
     def get_family_by_path(self, path: str) -> Optional[DocumentFamily]:
-        get_response = self._base_get(f"api/stores/{self.ref.replace(':', '/')}/fs",
-                                      params={"path": path, "meta": True})
-        return DocumentFamily.from_dict(get_response.json()) if get_response is not None else None
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(f"api/stores/{self.ref.replace(':', '/')}/fs",
+                                                       params={"path": path, "meta": True})
+        return DocumentFamily.parse_obj(get_response.json()) if get_response is not None else None
 
     def count(self) -> int:
-        get_response = self._base_get(f"api/stores/{self.ref.replace(':', '/')}/families")
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(f"api/stores/{self.ref.replace(':', '/')}/families")
         if get_response is not None:
             return get_response.json()['totalElements']
         else:
             return 0
 
     def get_by_content_object_id(self, document_family: DocumentFamily, content_object_id: str) -> Optional[Document]:
-        get_response = self._base_get(
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(
             f"api/stores/{self.ref.replace(':', '/')}/families/{document_family.id}/objects/{content_object_id}/content")
         if get_response is not None:
             return Document.from_kddb(get_response.content)
         else:
             return None
 
-    def _base_get(self, api_path: str, params: Optional[dict] = None) -> Optional[Response]:
-        if params is None:
-            params = {}
-        from kodexa import KodexaPlatform
 
-        url = f"{KodexaPlatform.get_url()}/{api_path}"
-        logger.info(f"Accessing {url}")
-
-        get_response = requests.get(
-            url,
-            params=params,
-            headers={"x-access-token": KodexaPlatform.get_access_token()})
-
-        if get_response.status_code == 200:
-            return get_response
-        elif get_response.status_code == 404:
-            return None
-        else:
-            msg = "Get failed [" + get_response.text + "], response " + str(get_response.status_code)
-            logger.warning(msg)
-            raise Exception(msg)
-
-
-class RemoteModelStore(ModelStore, RemoteStore):
+class RemoteModelStore(ModelStore):
     """
     A remote model store allows you to store artifacts from your model
     """
 
-    def to_dict(self):
-        """ """
-        return {
-            "type": "MODEL",
-            "ref": self.ref
-        }
-
-    def __init__(self, ref: str):
-        self.ref = ref
-
-    def get_ref(self) -> str:
-        """
-        Get the reference to the store on the platform (i.e. kodexa/my-store:1.1.0)
-
-        Returns:
-            The reference to the store
-        """
-        return self.ref
-
     def delete(self, object_path: str):
         """
         Delete the content stored in the model store at the given path
@@ -617,12 +487,13 @@ class RemoteModelStore(ModelStore, RemoteStore):
             logger.warning(msg)
             raise Exception(msg)
 
-    def put(self, path: str, content) -> DocumentFamily:
+    def put(self, path: str, content, replace=False) -> DocumentFamily:
         """Put the content into the model store at the given path
 
         Args:
           path: The path you wish to put the content at
           content: The content for that object
+          replace: Replace the content if it exists
 
         Returns:
           the document family that was created
@@ -631,6 +502,13 @@ class RemoteModelStore(ModelStore, RemoteStore):
         import requests
         try:
             files = {"file": content}
+
+            if replace:
+                requests.delete(
+                    f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/fs",
+                    params={"path": path},
+                    headers={"x-access-token": KodexaPlatform.get_access_token()})
+
             content_object_response = requests.post(
                 f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/fs",
                 params={"path": path},
@@ -638,7 +516,7 @@ class RemoteModelStore(ModelStore, RemoteStore):
                 files=files)
 
             if content_object_response.status_code == 200:
-                return DocumentFamily.from_dict(content_object_response.json())
+                return DocumentFamily.parse_obj(content_object_response.json())
             elif content_object_response.status_code == 400:
                 from addict import Dict
                 bad_request = Dict(json.loads(content_object_response.text))
@@ -667,7 +545,7 @@ class RemoteModelStore(ModelStore, RemoteStore):
             content_object_response = requests.put(
                 f"{KodexaPlatform.get_url()}/api/stores/{self.ref.replace(':', '/')}/metadata",
                 headers={"x-access-token": KodexaPlatform.get_access_token()},
-                json=model_content_metadata.to_dict())
+                json=model_content_metadata.dict())
 
             if content_object_response.status_code == 200:
                 return model_content_metadata
@@ -700,8 +578,27 @@ class RemoteModelStore(ModelStore, RemoteStore):
             headers={"x-access-token": KodexaPlatform.get_access_token()})
 
         if resp.status_code == 200:
-            return ModelContentMetadata.from_dict(resp.json())
+            return ModelContentMetadata.parse_obj(resp.json())
         else:
             msg = f"Unable to get model object {resp.text}, status : {resp.status_code}"
             logger.warning(msg)
             raise Exception(msg)
+
+    def list_contents(self) -> List[str]:
+
+        # TODO this needs to be cleaned up a bit
+        params = {
+            'page': 1,
+            'pageSize': 1000,
+            'query': '*'
+        }
+        from kodexa import KodexaPlatform
+        get_response = KodexaPlatform.get_client().get(f"api/stores/{self.ref.replace(':', '/')}/families",
+                                                       params=params)
+        if get_response is not None:
+            paths = []
+            for fam_dict in get_response.json()['content']:
+                paths.append(fam_dict['path'])
+            return paths
+        else:
+            return []
